Genesis.ETS (genesis v0.10.0)

Copy Markdown View Source

This module provides useful helpers for working with ETS tables.

Summary

Functions

Groups entries in the ETS table by their keys. Returns a stream of tuples containing the key and all values.

Streams all entries from the given ETS table.

Functions

group_keys(table)

Groups entries in the ETS table by their keys. Returns a stream of tuples containing the key and all values.

stream(table, transform \\ & &1)

Streams all entries from the given ETS table.