A struct that represents an entity and contains information about its origin. In essence, an entity is simply a unique identifier (reference) that exists within a specific context. An entity holds information about the world and context it belongs to, along with a unique identification hash.
Fields
:ref- the unique reference for this entity:context- the context where the entity lives:world- the world the entity was instantiated on:hash- a checksum that represents the entity's identity:name- the name the entity was registered with:parent- the entity this was cloned from
Summary
Functions
Returns true when the entity is a child of another entity.
Returns true when two entities share the same context.
Returns true when two entities represent the same identity.
Returns true when the entity was created on this node.
Returns true when the entity has a name.
Creates a new entity.
Returns true when the entity is a prefab.
Types
Functions
Returns true when the entity is a child of another entity.
Returns true when two entities share the same context.
Returns true when two entities represent the same identity.
Returns true when the entity was created on this node.
Returns true when the entity has a name.
Creates a new entity.
Options:
:context- the context PID where the entity lives (required):world- the world GenServer PID if entity belongs to a world (optional):name- an optional name for the entity (optional):parent- the parent entity this was cloned from (optional)
Returns true when the entity is a prefab.